@@ -1,7 +1,6 @@
 /*
     This file is part of darktable,
-    Copyright (C) 2009-2023 darktable developers.
-    Copyright (c) 2012 James C. McPherson
+    Copyright (C) 2009-2025 darktable developers.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,9 +15,7 @@
     You should have received a copy of the GNU General Public License
     along with darktable.  If not, see <http://www.gnu.org/licenses/>.
 */
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+
 #include "bauhaus/bauhaus.h"
 #include "common/colorspaces.h"
 #include "common/darktable.h"
@@ -110,8 +107,13 @@ const dt_action_def_t dt_action_def_modifiers
       _action_elements_modifiers,
       NULL, TRUE };
 
-void dt_control_init(dt_control_t *s)
+void dt_control_init(const gboolean withgui)
 {
+  dt_control_t *s = darktable.control = calloc(1, sizeof(dt_control_t));
+  dt_atomic_set_int(&s->running, DT_CONTROL_STATE_DISABLED);
+
+  if(!withgui) return;
+
   s->actions_global = (dt_action_t){ DT_ACTION_TYPE_GLOBAL,
     "global",
     C_("accel", "global"),
@@ -200,6 +202,9 @@ void dt_control_init(dt_control_t *s)
   s->confirm_mapping = TRUE;
   s->widget_definitions = g_ptr_array_new ();
   s->input_drivers = NULL;
+  dt_atomic_set_int(&s->quitting, 0);
+  dt_atomic_set_int(&s->pending_jobs, 0);
+  s->cups_started = FALSE;
 
   dt_action_define_fallback(DT_ACTION_TYPE_IOP, &dt_action_def_iop);
   dt_action_define_fallback(DT_ACTION_TYPE_LIB, &dt_action_def_lib);
@@ -213,33 +218,27 @@ void dt_control_init(dt_control_t *s)
   s->actions_modifiers = dt_action_define(&s->actions_global, NULL,
                                           N_("modifiers"), NULL, &dt_action_def_modifiers);
 
-  memset(s->vimkey, 0, sizeof(s->vimkey));
-  s->vimkey_cnt = 0;
-
   // same thread as init
   s->gui_thread = pthread_self();
 
   // s->last_expose_time = dt_get_wtime();
   s->log_pos = s->log_ack = 0;
-  s->log_busy = 0;
+  s->busy = 0;
   s->log_message_timeout_id = 0;
-  dt_pthread_mutex_init(&(s->log_mutex), NULL);
+  dt_pthread_mutex_init(&s->log_mutex, NULL);
 
   s->toast_pos = s->toast_ack = 0;
-  s->toast_busy = 0;
   s->toast_message_timeout_id = 0;
-  dt_pthread_mutex_init(&(s->toast_mutex), NULL);
 
   pthread_cond_init(&s->cond, NULL);
   dt_pthread_mutex_init(&s->cond_mutex, NULL);
   dt_pthread_mutex_init(&s->queue_mutex, NULL);
   dt_pthread_mutex_init(&s->res_mutex, NULL);
-  dt_pthread_mutex_init(&s->run_mutex, NULL);
-  dt_pthread_mutex_init(&(s->global_mutex), NULL);
-  dt_pthread_mutex_init(&(s->progress_system.mutex), NULL);
+  dt_pthread_mutex_init(&s->global_mutex, NULL);
+  dt_pthread_mutex_init(&s->progress_system.mutex, NULL);
 
   // start threads
-  dt_control_jobs_init(s);
+  dt_control_jobs_init();
 
   s->button_down = 0;
   s->button_down_which = 0;
@@ -259,80 +258,135 @@ void dt_control_allow_change_cursor()
 
 void dt_control_change_cursor(dt_cursor_t curs)
 {
-  if(!darktable.control->lock_cursor_shape)
+  GdkWindow *window = gtk_widget_get_window(dt_ui_main_window(darktable.gui->ui));
+  if(!darktable.control->lock_cursor_shape && window)
   {
-    GtkWidget *widget = dt_ui_main_window(darktable.gui->ui);
-    GdkCursor *cursor = gdk_cursor_new_for_display(gdk_display_get_default(), curs);
-    gdk_window_set_cursor(gtk_widget_get_window(widget), cursor);
+    GdkCursor *cursor = gdk_cursor_new_for_display(gdk_window_get_display(window), curs);
+    gdk_window_set_cursor(window, cursor);
     g_object_unref(cursor);
   }
 }
 
+/* Some implementation and how-to use notes about control->running
+  It is declared as a global gint with states defined in dt_control_state_t and may only be
+    written/read or modified via atomic intructions to avoid a protecting mutex.
+
+  We have gboolean dt_control_running() checking for a running control system all over the code.
+
+  dt_control_quit() is called whenever we want to close darktable.
+    It is triggered either by callbacks from the gui frontend (ctrl-q or click on close button)
+    or from lua.
+    By setting control->running to DT_CONTROL_STATE_CLEANUP here we
+      a) make sure the system recognizes this a not running any more and
+      b) leave a note about pending cleanup
+
+  dt_control_shutdown() is called when darktable closes, it checks for pending work (threads to be joined)
+    via DT_CONTROL_STATE_CLEANUP before doing so.
+*/
+
 gboolean dt_control_running()
 {
-  // FIXME: when shutdown, run_mutex is not inited anymore!
-  dt_control_t *s = darktable.control;
-  dt_pthread_mutex_lock(&s->run_mutex);
-  const gboolean running = s->running;
-  dt_pthread_mutex_unlock(&s->run_mutex);
-  return running;
+  return dt_atomic_get_int(&darktable.control->running) == DT_CONTROL_STATE_RUNNING;
 }
 
 void dt_control_quit()
 {
-  dt_gui_gtk_quit();
-  // thread safe quit, 1st pass:
-  dt_pthread_mutex_lock(&darktable.control->cond_mutex);
-  dt_pthread_mutex_lock(&darktable.control->run_mutex);
-  darktable.control->running = FALSE;
-  dt_pthread_mutex_unlock(&darktable.control->run_mutex);
-  dt_pthread_mutex_unlock(&darktable.control->cond_mutex);
+  // Make sure we proceed further only if control is running
+  if(!dt_control_running()) return;
 
-  gtk_main_quit();
+  dt_control_t *dc = darktable.control;
+  // make sure the rest is done only once
+  if(dt_atomic_exch_int(&dc->quitting, 1) == 1) return;
+
+#ifdef HAVE_PRINT
+  dt_printers_abort_discovery();
+  // Cups timeout could be pretty long, at least 30seconds
+  // but don't rely on cups returning correctly so a timeout
+  for(int i = 0; i < 40000 && !dc->cups_started; i++)
+    g_usleep(1000);
+#endif
+
+  // We test pending jobs vs 1 as we always accept one DT_JOB_QUEUE_SYSTEM_FG job
+  if(dt_control_jobs_pending() > 1)
+  {
+    dt_control_log("<span foreground='#FF0000' background='#000000'>%s</span>",
+                   _("darktable will be locked until background work has been done"));
+  }
+
+  for(int i = 0; i < 50 && (dt_control_jobs_pending() > 1); i++)
+  {
+    g_usleep(100000);
+    dt_gui_process_events();
+  }
+
+  dt_pthread_mutex_lock(&dc->cond_mutex);
+  // set the "pending cleanup work" flag to be handled in dt_control_shutdown()
+  dt_atomic_set_int(&dc->running, DT_CONTROL_STATE_CLEANUP);
+  dt_pthread_mutex_unlock(&dc->cond_mutex);
+
+  if(g_atomic_int_get(&darktable.gui_running))
+  {
+    dt_gui_gtk_quit();
+    gtk_main_quit();
+  }
 }
 
-void dt_control_shutdown(dt_control_t *s)
+void dt_control_shutdown()
 {
+  dt_control_t *s = darktable.control;
   dt_pthread_mutex_lock(&s->cond_mutex);
-  dt_pthread_mutex_lock(&s->run_mutex);
-  s->running = FALSE;
-  dt_pthread_mutex_unlock(&s->run_mutex);
-  dt_pthread_mutex_unlock(&s->cond_mutex);
+  const gboolean cleanup = dt_atomic_exch_int(&s->running, DT_CONTROL_STATE_DISABLED) == DT_CONTROL_STATE_CLEANUP;
   pthread_cond_broadcast(&s->cond);
+  dt_pthread_mutex_unlock(&s->cond_mutex);
 
+  int err = 0; // collect all joining errors
   /* first wait for gphoto device updater */
 #ifdef HAVE_GPHOTO2
-  pthread_join(s->update_gphoto_thread, NULL);
+   err = dt_pthread_join(s->update_gphoto_thread);
 #endif
+
+  if(!cleanup)
+    return;   // if not running there are no threads to join
+
+  dt_print(DT_DEBUG_CONTROL, "[dt_control_shutdown] closing control threads");
+
   /* then wait for kick_on_workers_thread */
-  pthread_join(s->kick_on_workers_thread, NULL);
+  err = dt_pthread_join(s->kick_on_workers_thread);
+  dt_print(DT_DEBUG_CONTROL, "[dt_control_shutdown] joined kicker%s", err ? ", error" : "");
 
-  int k;
-  for(k = 0; k < s->num_threads; k++)
-    // pthread_kill(s->thread[k], 9);
-    pthread_join(s->thread[k], NULL);
-  for(k = 0; k < DT_CTL_WORKER_RESERVED; k++)
-    // pthread_kill(s->thread_res[k], 9);
-    pthread_join(s->thread_res[k], NULL);
+  for(int k = 0; k < s->num_threads-1; k++)
+  {
+    err = dt_pthread_join(s->thread[k]);
+    dt_print(DT_DEBUG_CONTROL, "[dt_control_shutdown] joined num_thread %i%s", k, err ? ", error" : "");
+  }
 
+  for(int k = 0; k < DT_CTL_WORKER_RESERVED; k++)
+  {
+    err = dt_pthread_join(s->thread_res[k]);
+    dt_print(DT_DEBUG_CONTROL, "[dt_control_shutdown] joined worker %i%s", k, err ? ", error" : "");
+  }
 }
 
-void dt_control_cleanup(dt_control_t *s)
+void dt_control_cleanup(const gboolean withgui)
 {
-  // vacuum TODO: optional?
-  // DT_DEBUG_SQLITE3_EXEC(dt_database_get(darktable.db), "PRAGMA incremental_vacuum(0)", NULL, NULL, NULL);
-  // DT_DEBUG_SQLITE3_EXEC(dt_database_get(darktable.db), "vacuum", NULL, NULL, NULL);
-  dt_control_jobs_cleanup(s);
-  dt_pthread_mutex_destroy(&s->queue_mutex);
-  dt_pthread_mutex_destroy(&s->cond_mutex);
-  dt_pthread_mutex_destroy(&s->log_mutex);
-  dt_pthread_mutex_destroy(&s->toast_mutex);
-  dt_pthread_mutex_destroy(&s->res_mutex);
-  dt_pthread_mutex_destroy(&s->run_mutex);
-  dt_pthread_mutex_destroy(&s->progress_system.mutex);
-  if(s->widgets) g_hash_table_destroy(s->widgets);
-  if(s->shortcuts) g_sequence_free(s->shortcuts);
-  if(s->input_drivers) g_slist_free_full(s->input_drivers, g_free);
+  dt_control_t *s = darktable.control;
+  if(withgui)
+  {
+    // vacuum TODO: optional?
+    // DT_DEBUG_SQLITE3_EXEC(dt_database_get(darktable.db), "PRAGMA incremental_vacuum(0)", NULL, NULL, NULL);
+    // DT_DEBUG_SQLITE3_EXEC(dt_database_get(darktable.db), "vacuum", NULL, NULL, NULL);
+    dt_control_jobs_cleanup();
+    dt_pthread_mutex_destroy(&s->queue_mutex);
+    dt_pthread_mutex_destroy(&s->cond_mutex);
+    dt_pthread_mutex_destroy(&s->log_mutex);
+    dt_pthread_mutex_destroy(&s->res_mutex);
+    dt_pthread_mutex_destroy(&s->progress_system.mutex);
+    if(s->widgets) g_hash_table_destroy(s->widgets);
+    if(s->shortcuts) g_sequence_free(s->shortcuts);
+    if(s->input_drivers) g_slist_free_full(s->input_drivers, g_free);
+  }
+  free(s);
+  darktable.control = NULL;
 }
 
 
@@ -349,14 +403,6 @@ gboolean dt_control_configure(GtkWidget *da,
   return TRUE;
 }
 
-static GdkRGBA _lookup_color(GtkStyleContext *context, const char *name)
-{
-  GdkRGBA color, fallback = {1.0, 0.0, 0.0, 1.0};
-  if(!gtk_style_context_lookup_color (context, name, &color))
-    color = fallback;
-  return color;
-}
-
 void dt_control_draw_busy_msg(cairo_t *cr, int width, int height)
 {
   PangoRectangle ink;
@@ -389,58 +435,26 @@ void dt_control_draw_busy_msg(cairo_t *cr, int width, int height)
   g_object_unref(layout);
 }
 
-void *dt_control_expose(void *voidptr)
+void dt_control_expose(GtkWidget *widget, cairo_t *cr)
 {
   int pointerx, pointery;
-  if(!darktable.gui->surface) return NULL;
-  const int width = dt_cairo_image_surface_get_width(darktable.gui->surface);
-  const int height = dt_cairo_image_surface_get_height(darktable.gui->surface);
-  GtkWidget *widget = dt_ui_center(darktable.gui->ui);
   gdk_window_get_device_position(gtk_widget_get_window(widget),
       gdk_seat_get_pointer(gdk_display_get_default_seat(gtk_widget_get_display(widget))),
       &pointerx, &pointery, NULL);
 
-  // create a gtk-independent surface to draw on
-  cairo_surface_t *cst = dt_cairo_image_surface_create(CAIRO_FORMAT_ARGB32, width, height);
-  cairo_t *cr = cairo_create(cst);
+  dt_control_t *dc = darktable.control;
+  dc->width = gtk_widget_get_allocated_width(widget);
+  dc->height = gtk_widget_get_allocated_height(widget);
 
-  // TODO: control_expose: only redraw the part not overlapped by
-  // temporary control panel show!
-  //
-  darktable.control->width = width;
-  darktable.control->height = height;
-
-  GtkStyleContext *context = gtk_widget_get_style_context(widget);
-
-  // look up some colors once
-  const GdkRGBA bg_color = _lookup_color(context, "bg_color");
-
-  gdk_cairo_set_source_rgba(cr, &bg_color);
-  cairo_save(cr);
-  cairo_rectangle(cr, 0, 0, width, height);
-  cairo_clip(cr);
-  cairo_new_path(cr);
-  // draw view
-  dt_view_manager_expose(darktable.view_manager, cr, width, height, pointerx, pointery);
-  cairo_restore(cr);
+  dt_view_manager_expose(darktable.view_manager, cr, dc->width, dc->height, pointerx, pointery);
 
   // draw busy indicator
-  dt_pthread_mutex_lock(&darktable.control->log_mutex);
-  if(darktable.control->log_busy > 0)
+  dt_pthread_mutex_lock(&dc->log_mutex);
+  if(dc->busy > 0)
   {
-    dt_control_draw_busy_msg(cr, width, height);
+    dt_control_draw_busy_msg(cr, dc->width, dc->height);
   }
-  dt_pthread_mutex_unlock(&darktable.control->log_mutex);
-
-  cairo_destroy(cr);
-
-  cairo_t *cr_pixmap = cairo_create(darktable.gui->surface);
-  cairo_set_source_surface(cr_pixmap, cst, 0, 0);
-  cairo_paint(cr_pixmap);
-  cairo_destroy(cr_pixmap);
-
-  cairo_surface_destroy(cst);
-  return NULL;
+  dt_pthread_mutex_unlock(&dc->log_mutex);
 }
 
 gboolean dt_control_draw_endmarker(GtkWidget *widget,
@@ -455,7 +469,7 @@ gboolean dt_control_draw_endmarker(GtkWidget *widget,
   cairo_t *cr = cairo_create(cst);
   dt_draw_endmarker(cr, width, height, GPOINTER_TO_INT(user_data));
   cairo_destroy(cr);
-  cairo_set_source_surface(crf, cst, 0, 0);
+  cairo_set_source_surface(crf, cst, 0.0, 0.0);
   cairo_paint(crf);
   cairo_surface_destroy(cst);
   return TRUE;
@@ -494,7 +508,6 @@ static void _dt_ctl_switch_mode_prepare()
 {
   darktable.control->button_down = 0;
   darktable.control->button_down_which = 0;
-  darktable.gui->center_tooltip = 0;
   GtkWidget *widget = dt_ui_center(darktable.gui->ui);
   gtk_widget_set_tooltip_text(widget, "");
   gtk_widget_grab_focus(widget);
@@ -518,11 +531,11 @@ static gboolean _dt_ctl_switch_mode_to_by_view(gpointer user_data)
 
 void dt_ctl_switch_mode_to(const char *mode)
 {
-  const dt_view_t *current_view = dt_view_manager_get_current_view(darktable.view_manager);
-  if(current_view && !g_ascii_strcasecmp(mode, current_view->module_name))
+  const dt_view_t *cv = dt_view_manager_get_current_view(darktable.view_manager);
+  if(cv && !g_ascii_strcasecmp(mode, cv->module_name))
   {
     // if we are not in lighttable, we switch back to that view
-    if(g_ascii_strcasecmp(current_view->module_name, "lighttable"))
+    if(g_ascii_strcasecmp(cv->module_name, "lighttable"))
       dt_ctl_switch_mode_to("lighttable");
     return;
   }
@@ -539,8 +552,8 @@ void dt_ctl_switch_mode_to_by_view(const dt_view_t *view)
 
 void dt_ctl_switch_mode()
 {
-  const dt_view_t *view = dt_view_manager_get_current_view(darktable.view_manager);
-  const char *mode = (view && !strcmp(view->module_name, "lighttable"))
+  const dt_view_t *cv = dt_view_manager_get_current_view(darktable.view_manager);
+  const char *mode = (cv && !strcmp(cv->module_name, "lighttable"))
                       ? "darkroom"
                       : "lighttable";
   dt_ctl_switch_mode_to(mode);
@@ -548,22 +561,22 @@ void dt_ctl_switch_mode()
 
 static gboolean _dt_ctl_log_message_timeout_callback(gpointer data)
 {
-  dt_pthread_mutex_lock(&darktable.control->log_mutex);
-  if(darktable.control->log_ack != darktable.control->log_pos)
-    darktable.control->log_ack = (darktable.control->log_ack + 1) % DT_CTL_LOG_SIZE;
-  darktable.control->log_message_timeout_id = 0;
-  dt_pthread_mutex_unlock(&darktable.control->log_mutex);
+  dt_control_t *dc = darktable.control;
+  dt_pthread_mutex_lock(&dc->log_mutex);
+  dc->log_ack = dc->log_pos;
+  dc->log_message_timeout_id = 0;
+  dt_pthread_mutex_unlock(&dc->log_mutex);
   dt_control_log_redraw();
   return FALSE;
 }
 
 static gboolean _dt_ctl_toast_message_timeout_callback(gpointer data)
 {
-  dt_pthread_mutex_lock(&darktable.control->toast_mutex);
-  if(darktable.control->toast_ack != darktable.control->toast_pos)
-    darktable.control->toast_ack = (darktable.control->toast_ack + 1) % DT_CTL_TOAST_SIZE;
-  darktable.control->toast_message_timeout_id = 0;
-  dt_pthread_mutex_unlock(&darktable.control->toast_mutex);
+  dt_control_t *dc = darktable.control;
+  dt_pthread_mutex_lock(&dc->log_mutex);
+  dc->toast_ack = dc->toast_pos;
+  dc->toast_message_timeout_id = 0;
+  dt_pthread_mutex_unlock(&dc->log_mutex);
   dt_control_toast_redraw();
   return FALSE;
 }
@@ -575,48 +588,53 @@ void dt_control_button_pressed(double x,
                                int type,
                                uint32_t state)
 {
-  darktable.control->button_down = 1;
-  darktable.control->button_down_which = which;
-  darktable.control->button_type = type;
-  darktable.control->button_x = x;
-  darktable.control->button_y = y;
+  dt_control_t *dc = darktable.control;
+  dc->button_down = 1;
+  dc->button_down_which = which;
+  dc->button_type = type;
+  dc->button_x = x;
+  dc->button_y = y;
   // adding pressure to this data structure is not needed right
   // now. should the need ever arise: here is the place to do it :)
-  // const float wd = darktable.control->width;
-  const double ht = darktable.control->height;
+  // const float wd = dc->width;
+  const double ht = dc->height;
 
   // ack log message:
-  dt_pthread_mutex_lock(&darktable.control->log_mutex);
+  dt_pthread_mutex_lock(&dc->log_mutex);
   const double /*xc = wd/4.0-20,*/ yc = ht * 0.85 + 10.0;
-  if(darktable.control->log_ack != darktable.control->log_pos)
-    if(which == 1 /*&& x > xc - 10 && x < xc + 10*/ && y > yc - 10.0 && y < yc + 10.0)
+  if(dc->log_ack != dc->log_pos)
+  {
+    if(which == 1 && y > yc - 10.0 && y < yc + 10.0)
     {
-      if(darktable.control->log_message_timeout_id)
+      if(dc->log_message_timeout_id)
       {
-        g_source_remove(darktable.control->log_message_timeout_id);
-        darktable.control->log_message_timeout_id = 0;
+        g_source_remove(dc->log_message_timeout_id);
+        dc->log_message_timeout_id = 0;
       }
-      darktable.control->log_ack = (darktable.control->log_ack + 1) % DT_CTL_LOG_SIZE;
-      dt_pthread_mutex_unlock(&darktable.control->log_mutex);
+      dc->log_ack = dc->log_pos;
+      dt_pthread_mutex_unlock(&dc->log_mutex);
       return;
     }
-  dt_pthread_mutex_unlock(&darktable.control->log_mutex);
+  }
+  dt_pthread_mutex_unlock(&dc->log_mutex);
 
   // ack toast message:
-  dt_pthread_mutex_lock(&darktable.control->toast_mutex);
-  if(darktable.control->toast_ack != darktable.control->toast_pos)
-    if(which == 1 /*&& x > xc - 10 && x < xc + 10*/ && y > yc - 10.0 && y < yc + 10.0)
+  dt_pthread_mutex_lock(&dc->log_mutex);
+  if(dc->toast_ack != dc->toast_pos)
+  {
+    if(which == 1 && y > yc - 10.0 && y < yc + 10.0)
     {
-      if(darktable.control->toast_message_timeout_id)
+      if(dc->toast_message_timeout_id)
       {
-        g_source_remove(darktable.control->toast_message_timeout_id);
-        darktable.control->toast_message_timeout_id = 0;
+        g_source_remove(dc->toast_message_timeout_id);
+        dc->toast_message_timeout_id = 0;
       }
-      darktable.control->toast_ack = (darktable.control->toast_ack + 1) % DT_CTL_TOAST_SIZE;
-      dt_pthread_mutex_unlock(&darktable.control->toast_mutex);
+      dc->toast_ack = dc->toast_pos;
+      dt_pthread_mutex_unlock(&dc->log_mutex);
       return;
     }
-  dt_pthread_mutex_unlock(&darktable.control->toast_mutex);
+  }
+  dt_pthread_mutex_unlock(&dc->log_mutex);
 
   if(!dt_view_manager_button_pressed(darktable.view_manager, x, y,
                                      pressure, which, type, state))
@@ -632,57 +650,65 @@ static gboolean _redraw_center(gpointer user_data)
 
 void dt_control_log(const char *msg, ...)
 {
-  dt_pthread_mutex_lock(&darktable.control->log_mutex);
+  if(!dt_control_running()) return;
+  dt_control_t *dc = darktable.control;
+  dt_pthread_mutex_lock(&dc->log_mutex);
   va_list ap;
   va_start(ap, msg);
   char *escaped_msg = g_markup_vprintf_escaped(msg, ap);
   const int msglen = strlen(escaped_msg);
-  g_strlcpy(darktable.control->log_message[darktable.control->log_pos],
-            escaped_msg, DT_CTL_LOG_MSG_SIZE);
+
+  const int old_idx = (dc->log_pos - 1) & (DT_CTL_LOG_SIZE-1);
+  const gboolean timeout = dc->log_message_timeout_id;
+  if(!timeout || g_strcmp0(escaped_msg, dc->log_message[old_idx]))
+  {
+    g_strlcpy(dc->log_message[dc->log_pos & (DT_CTL_LOG_SIZE-1)], escaped_msg, DT_CTL_LOG_MSG_SIZE);
+    dc->log_pos++;
+  }
+
   g_free(escaped_msg);
   va_end(ap);
-  if(darktable.control->log_message_timeout_id)
-    g_source_remove(darktable.control->log_message_timeout_id);
-  darktable.control->log_ack = darktable.control->log_pos;
-  darktable.control->log_pos = (darktable.control->log_pos + 1) % DT_CTL_LOG_SIZE;
 
-  darktable.control->log_message_timeout_id
+  if(timeout)
+    g_source_remove(dc->log_message_timeout_id);
+
+  dc->log_message_timeout_id
     = g_timeout_add(DT_CTL_LOG_TIMEOUT + 1000 * (msglen / 40),
                     _dt_ctl_log_message_timeout_callback, NULL);
-  dt_pthread_mutex_unlock(&darktable.control->log_mutex);
+  dt_pthread_mutex_unlock(&dc->log_mutex);
   // redraw center later in gui thread:
   g_idle_add(_redraw_center, 0);
 }
 
 static void _toast_log(const gboolean markup, const char *msg, va_list ap)
 {
-  dt_pthread_mutex_lock(&darktable.control->toast_mutex);
+  dt_control_t *dc = darktable.control;
+  dt_pthread_mutex_lock(&dc->log_mutex);
 
   // if we don't want markup, we escape <>&... so they are not interpreted later
   if(markup)
-    vsnprintf(darktable.control->toast_message[darktable.control->toast_pos],
-              DT_CTL_TOAST_MSG_SIZE, msg, ap);
+    vsnprintf(dc->toast_message[dc->toast_pos & (DT_CTL_TOAST_SIZE-1)], DT_CTL_TOAST_MSG_SIZE, msg, ap);
   else
   {
     char *escaped_msg = g_markup_vprintf_escaped(msg, ap);
-    g_strlcpy(darktable.control->toast_message[darktable.control->toast_pos],
-              escaped_msg, DT_CTL_TOAST_MSG_SIZE);
+    g_strlcpy(dc->toast_message[dc->toast_pos & (DT_CTL_TOAST_SIZE-1)], escaped_msg, DT_CTL_TOAST_MSG_SIZE);
     g_free(escaped_msg);
   }
+  dc->toast_pos++;
 
-  if(darktable.control->toast_message_timeout_id)
-    g_source_remove(darktable.control->toast_message_timeout_id);
-  darktable.control->toast_ack = darktable.control->toast_pos;
-  darktable.control->toast_pos = (darktable.control->toast_pos + 1) % DT_CTL_TOAST_SIZE;
-  darktable.control->toast_message_timeout_id
+  if(dc->toast_message_timeout_id)
+    g_source_remove(dc->toast_message_timeout_id);
+
+  dc->toast_message_timeout_id
       = g_timeout_add(DT_CTL_TOAST_TIMEOUT, _dt_ctl_toast_message_timeout_callback, NULL);
-  dt_pthread_mutex_unlock(&darktable.control->toast_mutex);
+  dt_pthread_mutex_unlock(&dc->log_mutex);
   // redraw center later in gui thread:
   g_idle_add(_redraw_center, 0);
 }
 
 void dt_toast_log(const char *msg, ...)
 {
+  if(!dt_control_running()) return;
   va_list ap;
   va_start(ap, msg);
   _toast_log(FALSE, msg, ap);
@@ -691,75 +717,58 @@ void dt_toast_log(const char *msg, ...)
 
 void dt_toast_markup_log(const char *msg, ...)
 {
+  if(!dt_control_running()) return;
   va_list ap;
   va_start(ap, msg);
   _toast_log(TRUE, msg, ap);
   va_end(ap);
 }
 
-static void _control_log_ack_all()
+void dt_control_busy_enter()
 {
-  dt_pthread_mutex_lock(&darktable.control->log_mutex);
-  darktable.control->log_pos = darktable.control->log_ack;
-  dt_pthread_mutex_unlock(&darktable.control->log_mutex);
-  dt_control_queue_redraw_center();
+  if(!dt_control_running()) return;
+  dt_control_t *dc = darktable.control;
+  dt_pthread_mutex_lock(&dc->log_mutex);
+  dc->busy++;
+  dt_pthread_mutex_unlock(&dc->log_mutex);
 }
 
-void dt_control_log_busy_enter()
+void dt_control_busy_leave()
 {
-  dt_pthread_mutex_lock(&darktable.control->log_mutex);
-  darktable.control->log_busy++;
-  dt_pthread_mutex_unlock(&darktable.control->log_mutex);
-}
-
-void dt_control_toast_busy_enter()
-{
-  dt_pthread_mutex_lock(&darktable.control->toast_mutex);
-  darktable.control->toast_busy++;
-  dt_pthread_mutex_unlock(&darktable.control->toast_mutex);
-}
-
-void dt_control_log_busy_leave()
-{
-  dt_pthread_mutex_lock(&darktable.control->log_mutex);
-  darktable.control->log_busy--;
-  dt_pthread_mutex_unlock(&darktable.control->log_mutex);
-  /* lets redraw */
-  dt_control_queue_redraw_center();
-}
-
-void dt_control_toast_busy_leave()
-{
-  dt_pthread_mutex_lock(&darktable.control->toast_mutex);
-  darktable.control->toast_busy--;
-  dt_pthread_mutex_unlock(&darktable.control->toast_mutex);
+  if(!dt_control_running()) return;
+  dt_control_t *dc = darktable.control;
+  dt_pthread_mutex_lock(&dc->log_mutex);
+  dc->busy--;
+  dt_pthread_mutex_unlock(&dc->log_mutex);
   /* lets redraw */
   dt_control_queue_redraw_center();
 }
 
 void dt_control_queue_redraw()
 {
-  DT_DEBUG_CONTROL_SIGNAL_RAISE(darktable.signals, DT_SIGNAL_CONTROL_REDRAW_ALL);
+  DT_CONTROL_SIGNAL_RAISE(DT_SIGNAL_CONTROL_REDRAW_ALL);
 }
 
 void dt_control_queue_redraw_center()
 {
-  DT_DEBUG_CONTROL_SIGNAL_RAISE(darktable.signals, DT_SIGNAL_CONTROL_REDRAW_CENTER);
+  DT_CONTROL_SIGNAL_RAISE(DT_SIGNAL_CONTROL_REDRAW_CENTER);
 }
 
 void dt_control_navigation_redraw()
 {
-  DT_DEBUG_CONTROL_SIGNAL_RAISE(darktable.signals, DT_SIGNAL_CONTROL_NAVIGATION_REDRAW);
+  DT_CONTROL_SIGNAL_RAISE(DT_SIGNAL_CONTROL_NAVIGATION_REDRAW);
 }
 
 void dt_control_log_redraw()
 {
-  DT_DEBUG_CONTROL_SIGNAL_RAISE(darktable.signals, DT_SIGNAL_CONTROL_LOG_REDRAW);
+  if(dt_control_running())
+    DT_CONTROL_SIGNAL_RAISE(DT_SIGNAL_CONTROL_LOG_REDRAW);
 }
 
 void dt_control_toast_redraw()
 {
-  DT_DEBUG_CONTROL_SIGNAL_RAISE(darktable.signals, DT_SIGNAL_CONTROL_TOAST_REDRAW);
+  if(dt_control_running())
+    DT_CONTROL_SIGNAL_RAISE(DT_SIGNAL_CONTROL_TOAST_REDRAW);
 }
 
 static int _widget_queue_draw(void *widget)
@@ -778,166 +787,36 @@ void dt_control_queue_redraw_widget(GtkWidget *widget)
   }
 }
 
-int dt_control_key_pressed_override(guint key, guint state)
+void dt_control_hinter_message(const char *message)
 {
-#ifdef HAVE_GAME
-  // ↑ ↑ ↓ ↓ ← → ← → b a
-  static int konami_state = 0;
-  static guint konami_sequence[] = {
-    GDK_KEY_Up,
-    GDK_KEY_Up,
-    GDK_KEY_Down,
-    GDK_KEY_Down,
-    GDK_KEY_Left,
-    GDK_KEY_Right,
-    GDK_KEY_Left,
-    GDK_KEY_Right,
-    GDK_KEY_b,
-    GDK_KEY_a
-  };
-  if(key == konami_sequence[konami_state])
-  {
-    konami_state++;
-    if(konami_state == G_N_ELEMENTS(konami_sequence))
-    {
-      dt_ctl_switch_mode_to("knight");
-      konami_state = 0;
-    }
-  }
-  else
-    konami_state = 0;
-#endif
-
-  // TODO: if darkroom mode
-  // did a : vim-style command start?
-  static GList *autocomplete = NULL;
-  if(darktable.control->vimkey_cnt)
-  {
-    gunichar unichar = gdk_keyval_to_unicode(key);
-    if(key == GDK_KEY_Return)
-    {
-      if(!strcmp(darktable.control->vimkey, ":q"))
-      {
-        dt_control_quit();
-      }
-      else
-      {
-        dt_bauhaus_vimkey_exec(darktable.control->vimkey);
-      }
-      darktable.control->vimkey[0] = 0;
-      darktable.control->vimkey_cnt = 0;
-      _control_log_ack_all();
-      g_list_free(autocomplete);
-      autocomplete = NULL;
-    }
-    else if(key == GDK_KEY_Escape)
-    {
-      darktable.control->vimkey[0] = 0;
-      darktable.control->vimkey_cnt = 0;
-      _control_log_ack_all();
-      g_list_free(autocomplete);
-      autocomplete = NULL;
-    }
-    else if(key == GDK_KEY_BackSpace)
-    {
-      darktable.control->vimkey_cnt
-          -= (darktable.control->vimkey + darktable.control->vimkey_cnt)
-             - g_utf8_prev_char(darktable.control->vimkey + darktable.control->vimkey_cnt);
-      darktable.control->vimkey[darktable.control->vimkey_cnt] = 0;
-      if(darktable.control->vimkey_cnt == 0)
-        _control_log_ack_all();
-      else
-        dt_control_log("%s", darktable.control->vimkey);
-      g_list_free(autocomplete);
-      autocomplete = NULL;
-    }
-    else if(key == GDK_KEY_Tab)
-    {
-      // TODO: also support :preset and :get?
-      // auto complete:
-      if(darktable.control->vimkey_cnt < 5)
-      {
-        g_strlcpy(darktable.control->vimkey, ":set ", sizeof(darktable.control->vimkey));
-        darktable.control->vimkey_cnt = 5;
-      }
-      else if(!autocomplete)
-      {
-        // TODO: handle '.'-separated things separately
-        // this is a static list, and tab cycles through the list
-        if(darktable.control->vimkey_cnt < strlen(darktable.control->vimkey))
-          darktable.control->vimkey[darktable.control->vimkey_cnt] = 0;
-        else
-          autocomplete = dt_bauhaus_vimkey_complete(darktable.control->vimkey + 5);
-      }
-      if(autocomplete)
-      {
-        // pop first.
-        // the paths themselves are owned by bauhaus,
-        // no free required.
-        darktable.control->vimkey[darktable.control->vimkey_cnt] = 0;
-        g_strlcat(darktable.control->vimkey, (char *)autocomplete->data,
-                  sizeof(darktable.control->vimkey));
-        autocomplete = g_list_remove(autocomplete, autocomplete->data);
-      }
-      dt_control_log("%s", darktable.control->vimkey);
-    }
-    else if(g_unichar_isprint(unichar)) // printable unicode character
-    {
-      gchar utf8[6] = { 0 };
-      g_unichar_to_utf8(unichar, utf8);
-      g_strlcat(darktable.control->vimkey, utf8, sizeof(darktable.control->vimkey));
-      darktable.control->vimkey_cnt = strlen(darktable.control->vimkey);
-      dt_control_log("%s", darktable.control->vimkey);
-      g_list_free(autocomplete);
-      autocomplete = NULL;
-    }
-    else if(key == GDK_KEY_Up)
-    {
-      // TODO: step history up and copy to vimkey
-    }
-    else if(key == GDK_KEY_Down)
-    {
-      // TODO: step history down and copy to vimkey
-    }
-    return 1;
-  }
-  else if(key == ':')
-  {
-    darktable.control->vimkey[0] = ':';
-    darktable.control->vimkey[1] = 0;
-    darktable.control->vimkey_cnt = 1;
-    dt_control_log("%s", darktable.control->vimkey);
-    return 1;
-  }
-
-  return 0;
-}
-
-void dt_control_hinter_message(const struct dt_control_t *s, const char *message)
-{
-  if(s->proxy.hinter.module)
+  if(!dt_control_running())
+    return;
+  dt_control_t *s = darktable.control;
+  if(s && s->proxy.hinter.module)
     return s->proxy.hinter.set_message(s->proxy.hinter.module, message);
 }
 
 dt_imgid_t dt_control_get_mouse_over_id()
 {
-  dt_pthread_mutex_lock(&(darktable.control->global_mutex));
-  const dt_imgid_t result = darktable.control->mouse_over_id;
-  dt_pthread_mutex_unlock(&(darktable.control->global_mutex));
+  dt_control_t *dc = darktable.control;
+  dt_pthread_mutex_lock(&dc->global_mutex);
+  const dt_imgid_t result = dc->mouse_over_id;
+  dt_pthread_mutex_unlock(&dc->global_mutex);
   return result;
 }
 
 void dt_control_set_mouse_over_id(const dt_imgid_t imgid)
 {
-  dt_pthread_mutex_lock(&(darktable.control->global_mutex));
-  if(darktable.control->mouse_over_id != imgid)
+  dt_control_t *dc = darktable.control;
+  dt_pthread_mutex_lock(&dc->global_mutex);
+  if(dc->mouse_over_id != imgid)
   {
-    darktable.control->mouse_over_id = imgid;
-    dt_pthread_mutex_unlock(&(darktable.control->global_mutex));
-    DT_DEBUG_CONTROL_SIGNAL_RAISE(darktable.signals, DT_SIGNAL_MOUSE_OVER_IMAGE_CHANGE);
+    dc->mouse_over_id = imgid;
+    dt_pthread_mutex_unlock(&dc->global_mutex);
+    DT_CONTROL_SIGNAL_RAISE(DT_SIGNAL_MOUSE_OVER_IMAGE_CHANGE);
   }
   else
-    dt_pthread_mutex_unlock(&(darktable.control->global_mutex));
+    dt_pthread_mutex_unlock(&dc->global_mutex);
 }
 
 // clang-format off
